//-*-c++-*------------------------------------------------------------
//
// File name : bioArithAbs.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 10:56:09 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"

#include "bioArithAbs.h"
#include "bioArithConstant.h"

bioArithAbs::bioArithAbs(bioExpression* par,
                         bioExpression* left) : bioExpression(par,left,NULL)
{
}

bioArithAbs::~bioArithAbs() {}

bioExpression::bioOperatorType bioArithAbs::getOperatorType() const {
  return bioExpression::UNARY_OP ;
}

patString bioArithAbs::getOperatorName() const {
  return ("abs") ;
}

patReal bioArithAbs::getValue(patError*& err) const  {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal result = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }

  return ((result>=0)?result:-result) ;
}


bioExpression* bioArithAbs::getDerivative(patULong aLiteralId, patError*& err) const {
  if (dependsOf(aLiteralId)) {
    err = new patErrMiscError("No derivative for boolean functions") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  else {
    bioExpression* result = new bioArithConstant(NULL,0.0) ;
    return result ;
  }
}

bioArithAbs* bioArithAbs::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
  }
  if (rightChild != NULL) { // Must be NULL
    rightClone = rightChild->getDeepCopy(err) ;
  }

  bioArithAbs* theNode = 
    new bioArithAbs(NULL,leftClone) ;
  leftClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithAbs::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getCppCode(prefix, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << vLeft << endl ;
  str << getCppResultDeclaration(prefix) << " = patAbs(" << leftChild->getCppResult(prefix) << ");" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }

}

patString bioArithAbs::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "patAbs(" << vLeft << ")" ;
  return patString(str.str()) ;
}

patString bioArithAbs::getExpressionString() const {
  stringstream str ;
  str << "$A" ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}
