//-*-c++-*------------------------------------------------------------
//
// File name : bioArithExp.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 14:14:41 2009
//
//--------------------------------------------------------------------


#include <math.h>
#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "bioArithExp.h"
#include "bioArithMult.h"

bioArithExp::bioArithExp(bioExpression* par,
                         bioExpression* left) 
    : bioExpression(par,left,NULL)
{
}

bioArithExp::~bioArithExp() {}

bioExpression::bioOperatorType bioArithExp::getOperatorType() const {
  return bioExpression::UNARY_OP ;
}

patString bioArithExp::getOperatorName() const {
  return ("exp") ;
}

patReal bioArithExp::getValue(patError*& err) const  {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal result = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  
  if (!finite(result) || result >= patLogMaxReal::the()) {
    WARNING("Try to compute exp(" << result << ")") ;
    return patMaxReal ;
  }
  else {
    return exp(result) ;
  }
}


bioExpression* bioArithExp::getDerivative(patULong aLiteralId, patError*& err) const {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* value = leftChild->getDeepCopy(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  bioExpression* deriv = leftChild->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* e = new bioArithExp(NULL, value) ;
  bioExpression* result = new bioArithMult(NULL,deriv,e) ;
  return result ;
}


bioArithExp* bioArithExp::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  //bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
  }
  //if (rightChild != NULL) {
  //  rightClone = rightChild->getDeepCopy(err) ;
  //}

  bioArithExp* theNode = 
    new bioArithExp(NULL,leftClone) ;
  leftClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithExp::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getCppCode(prefix, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }


  stringstream str ;
  str << vLeft << endl ;
  str << getCppResultDeclaration(prefix) << " = patExp(" << leftChild->getCppResult(prefix) << ");" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;

  
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithExp::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "patExp(" << vLeft << ")" ;
  return patString(str.str()) ;
}

patString bioArithExp::getExpressionString() const {
  stringstream str ;
  str << '$E' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}
