//-*-c++-*------------------------------------------------------------
//
// File name : bioArithLog.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 14:05:01 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "bioArithLog.h"
#include "bioArithDivide.h"

bioArithLog::bioArithLog(bioExpression* par,
                         bioExpression* left) 
    : bioExpression(par,left,NULL)
{
}

bioArithLog::~bioArithLog() {}

bioExpression::bioOperatorType bioArithLog::getOperatorType() const {
  return bioExpression::UNARY_OP ;
}

patString bioArithLog::getOperatorName() const {
  return ("log") ;
}

patReal bioArithLog::getValue(patError*& err) const  {
  if (leftChild == NULL) {
    err = new patErrNullPointer("patArithNode") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal result = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }

  return log(result) ;
}


bioExpression* bioArithLog::getDerivative(patULong aLiteralId, patError*& err) const {
  if (leftChild == NULL) {
    err = new patErrNullPointer("patArithNode") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  
  bioExpression* value = leftChild->getDeepCopy(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* deriv = leftChild->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  bioExpression* result = new bioArithDivide(NULL,deriv,value) ;
  return result ;
}


bioArithLog* bioArithLog::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
  }
  if (rightChild != NULL) { // Must be NULL
    rightClone = rightChild->getDeepCopy(err) ;
  }

  bioArithLog* theNode = 
    new bioArithLog(NULL,leftClone) ;
  leftClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithLog::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getCppCode(prefix, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }


  stringstream str ;
  str << vLeft << endl ;
  str << getCppResultDeclaration(prefix) << " ;" << endl ;
  str << "if (!finite(log(" << leftChild->getCppResult(prefix) <<  "))) {" << endl ;
  
  //  str << "if (" << leftChild->getCppResult(prefix) << " <= " << patEPSILON << ") {" << endl ;
  str << "  " << getCppResult(prefix)   << "= " << -patMaxReal/2.0 << ";" << endl ;
  str << "}" << endl ;
  str << "else {" << endl ;
  str << "   " << getCppResult(prefix) << " = log(" << leftChild->getCppResult(prefix) << ");" << endl ;
  str << "}" << endl ;
  
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
 
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithLog::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "log(" << vLeft << ")" ;
  return patString(str.str()) ;
}

patBoolean bioArithLog::isSimple() const {
  return patFALSE ;
}

patString bioArithLog::getExpressionString() const {
  stringstream str ;
  str << '$L' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}

