//-*-c++-*------------------------------------------------------------
//
// File name : bioArithMergedSum.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Thu Aug  6 14:11:39 2009
//
//--------------------------------------------------------------------

#ifndef bioArithMergedSum_h
#define bioArithMergedSum_h

#include "bioExpression.h"

class bioIteratorInfo ;

/*!
This class is designed to optimize the generated code. It merges within one loops various expressions based on the same iterator.
*/

class bioArithMergedSum: public bioExpression {
 public:
  bioArithMergedSum(bioExpression* par,
		    bioIteratorInfo* aIterator,
		    vector<patString> results,
		    vector<bioExpression*> listOfExpressions) ;
  ~bioArithMergedSum() ;

  virtual bioOperatorType getOperatorType() const ;
  virtual patString getOperatorName() const;
  virtual patReal getValue(patError*& err) const  ;
  virtual bioExpression* getDerivative(patULong aLiteralId, 
				       patError*& err) const ;
  virtual bioArithMergedSum* getDeepCopy(patError*& err) ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual patString getSimpleCppCode(patError*& err) ;
  virtual patString getExpression(patError*& err) const ;
  virtual patBoolean isSimple() const ;
  virtual patBoolean dependsOf(patULong aLiteralId) const ;
  virtual void forceCodeGeneration() ;
  virtual void simplifyZeros(patError*& err) ;
  virtual bioIteratorInfo* theIterator() ;
  virtual patULong getNumberOfOperations() const ;
  virtual patString getIncludeStatements(patString prefix) const ;
  virtual patBoolean containsAnIterator() const ;
  virtual patBoolean containsAnIntegral() const ;
  virtual patBoolean containsASequence() const ;
  virtual void generateGslFunctions(ostream& str_h,patString prefix,patError*& err) ;
  virtual patULong countExpressions(map<patString,patULong>* theExpressions, 
				    map<patString,list<patULong> >* theIds,
				    map<patString,list<bioExpression*> >* thePointers,
				    patError*& err)   ;
  void replaceExpression(bioExpression* theExpression, 
			 bioExpression* anotherExpression, 
			 patError*& err) ;
  virtual bioExpression* getExpressionFromId(patULong theId) ;
  virtual patString getExpressionString() const ;
  virtual void getExpressionsToSimplify(vector<bioExpression*>* theExpr, 
				patError*& err) ;

 private:
  void tryToSimplify(patError*& err) ;
  bioIteratorInfo* theIteratorInfo ;
  vector<patString> variablesNames ;
  vector<bioExpression*> allExpressions ;
  vector<bioExpression*> repeatedExpressions ;
  vector<bioExpression*> shortExpressions ;
  
};
#endif
