//-*-c++-*------------------------------------------------------------
//
// File name : bioArithOr.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 10:36:07 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"

#include "bioArithOr.h"
#include "bioArithConstant.h"

bioArithOr::bioArithOr(bioExpression* par,
                       bioExpression* left, 
                       bioExpression* right) : bioExpression(par,left,right)
{
}

bioArithOr::~bioArithOr() {}

bioExpression::bioOperatorType bioArithOr::getOperatorType() const {
  return bioExpression::BINARY_OP ;
}

patString bioArithOr::getOperatorName() const {
  return ("or") ;
}

patReal bioArithOr::getValue(patError*& err) const  {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal left = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  patReal right = rightChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  patReal result = left || right ;

  return result ;
}

bioExpression* bioArithOr::getDerivative(patULong aLiteralId, patError*& err) const {
  if (dependsOf(aLiteralId)) {
    err = new patErrMiscError("No derivative for boolean functions") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  else {
    bioExpression* result = new bioArithConstant(NULL,0.0) ;
    return result ;
  }
}

bioArithOr* bioArithOr::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (leftClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  if (rightChild != NULL) {
    rightClone = rightChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (rightClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }

  bioArithOr* theNode = 
    new bioArithOr(NULL,leftClone,rightClone) ;
  leftClone->setParent(theNode) ;
  rightClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithOr::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
    if (leftChild == NULL || rightChild == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return patString();
    }
  
    patBoolean lSimple = leftChild->isSimple() ;
    patBoolean rSimple = rightChild->isSimple() ;
    patString vLeft ;
    if (lSimple) {
      vLeft = leftChild->getSimpleCppCode(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patString();
      }
    }
    else {
      vLeft = leftChild->getCppCode(prefix, err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patString();
      }
    }
    patString vRight ;
    if (rSimple) {
      vRight = rightChild->getSimpleCppCode(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patString();
      }
    }
    else {
      vRight = rightChild->getCppCode(prefix, err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patString();
      }
    }
    stringstream str ;
    if (!lSimple) {
      str << vLeft << endl ;
    }
    if (!rSimple) {
      str << vRight << endl ;
    }
    str << getCppResultDeclaration(prefix) << " = (" ;
    if (lSimple) {
      str << vLeft ;
    }
    else {
      str << leftChild->getCppResult(prefix) ;
    }
    str << ") + ( " ;
    if (rSimple) {
      str << vRight ;    
    }
    else {
      str << rightChild->getCppResult(prefix) ;
    }
    str << ");" << endl ;
    //    str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
    codeMustBeGenerated = patFALSE ;
    return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}


patString bioArithOr::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  patString vRight = rightChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "(" << vLeft << ")+(" << vRight << ")" ;
  return patString(str.str()) ;
}



patString bioArithOr::getExpressionString() const {
  stringstream str ;
  str << '|' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  if (rightChild != NULL) {
    str << '{' << rightChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;

}
