//-*-c++-*------------------------------------------------------------
//
// File name : bioArithPrint.cc
// Author :    Michel Bierlaire
// Date :      Tue May 11 07:10:16 2010
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patError.h"

#include "bioArithPrint.h"
#include "bioArithBinaryPlus.h"
#include "bioLiteralRepository.h"
#include "bioIteratorInfoRepository.h"

/*!
*/
bioArithPrint::bioArithPrint(bioExpression* par,
			     map<patString,bioExpression*> theTerms,
			     bioIteratorInfo * it) 
  : bioArithIterator(par,NULL,it),theExpressions(theTerms) {

}

bioArithPrint::~bioArithPrint() {}

bioExpression::bioOperatorType bioArithPrint::getOperatorType() const {
  return bioExpression::ENUM_OP ;
}

patString bioArithPrint::getOperatorName() const {
  return patString("Print") ;
}

bioExpression* bioArithPrint::getDerivative(patULong aLiteralId, patError*& err) const {

  err = new patErrMiscError("No derivative for the print expression") ;
  WARNING(err->describe()) ;
  return NULL;
}


bioArithPrint* bioArithPrint::getDeepCopy(patError*& err) {

  map<patString,bioExpression*> newDict(theExpressions) ;

  bioArithPrint* theNode = 
    new bioArithPrint(NULL,newDict,theIteratorInfo) ;
  
  return theNode ;
}



patBoolean bioArithPrint::isSum() const {
  return patFALSE ;
}

patBoolean bioArithPrint::isProd() const {
  return patFALSE ;
}

patString bioArithPrint::getExpressionString() const {
  stringstream str ;
  str << '$Print' ;
  str << theIteratorInfo->iteratorName ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}

patString bioArithPrint::getExpression(patError*& err) const {
  stringstream ss ;
  ss << theIteratorInfo->getInfo() ;
  ss << "[" ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    if (i != theExpressions.begin()) {
      ss << ", " ;
    }
    ss << i->first << "=" << *(i->second) ;
  }
  ss << "]" ;

  return patString(ss.str()) ;
  
}

patULong bioArithPrint::getNumberOfOperations() const {
  patULong count = 0 ;
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    count += i->second->getNumberOfOperations() ;
  }
  return(count) ;

}

patString bioArithPrint::getCppCode(patString prefix, patError*& err) {
  DEBUG_MESSAGE("getCPpCode"); 
  if (codeMustBeGenerated) {
    stringstream str ;
    str << endl << "//" << endl << "// " << *this << endl << "//" << endl ;
    if (theIteratorInfo == NULL) {
      err = new patErrNullPointer("bioIteratorInfo") ;
      WARNING(err->describe()) ;
      return patString() ;
    }


    str << getCppResultDeclaration(prefix) << " = 0.0 ;" << endl ;

    patString cppType = theIteratorInfo->getCppType(err) ;
    patString n = theIteratorInfo->iteratorName ;
    if (err != NULL) {
      WARNING(err->describe()) ;      return patString() ;
    }

    // Make sure that the expressions are regenerated at least once
    // within the loop.
    
    for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
	 i != theExpressions.end() ;
	 ++i) {
      i->second->forceCodeGeneration() ;
    }
    bioLiteralRepository::the()->resetAllFlags() ;


    stringstream i ;
    i << getId() ;
    patString uniqueId(i.str()) ;
    theIteratorInfo->idOfCurrentNodeInExpression = uniqueId ;

    if (theIteratorInfo->isTopIterator()) {
    
      str << "bioIteratorSpan " << prefix << "aSpan" << uniqueId << "(patString(\"" << n << "\"),0) ;" << endl ;
    }
    if (theIteratorInfo->isMetaIterator()) {
      str << " bioMetaIterator* " << prefix << "theIter_" << n << "_" << uniqueId << "  = sample->createMetaIterator(" << prefix << "aSpan" ;
      if (theIteratorInfo->isTopIterator()) {
	str << uniqueId; 
      }
      else {
	str << theIteratorInfo->theParent->idOfCurrentNodeInExpression ;
      }
      str <<",__theThreadSpan,err) ;" << endl ;
      str << "      if (err != NULL) {" << endl ;
      str << "	WARNING(err->describe()) ;" << endl ;
      str << " exit(-1) ;" << endl ;
      str << "      }" << endl ;
    }
    else if (theIteratorInfo->isRowIterator()) {
      //      DEBUG_MESSAGE("Create code for row iterator: " << *theIteratorInfo) ;
      bioIteratorInfo* theChild = bioIteratorInfoRepository::the()->getIteratorInfo(theIteratorInfo->childIteratorName) ;
      str << " bioRowIterator* " << prefix << "theIter_" << n <<  "_" << uniqueId<< "  = sample->createRowIterator(" << prefix << "aSpan" ;
      if (theIteratorInfo->isTopIterator()) {
	str << uniqueId; 
      }
      else {
	str << theIteratorInfo->theParent->idOfCurrentNodeInExpression ;
      }
      str <<",__theThreadSpan,err) ;" << endl ;
      str << "      if (err != NULL) {" << endl ;
      str << "	WARNING(err->describe()) ;" << endl ;
      str << "	exit(-1) ;" << endl ;
      str << "      }" << endl ;
    }
    else if (theIteratorInfo->isDrawIterator()) {
      str << " bioDrawIterator* " << prefix << "theIter_" << n <<  "_" << uniqueId << "  = bioRandomDraws::the()->createIterator() ;" << endl ;
      
    }
    else {
      stringstream str ;
      str << "Unknown iterator: " << *theIteratorInfo ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()) ;
      return patString();
    }

 
    // Open the file

    str << "    patString htmlFileName = patFileNames::the()->getHtmlFile(err) ;" << endl ;
    str << "    if (err != NULL) {" << endl ;
    str << "      WARNING(err->describe()) ;" << endl ;
    str << "      exit(-1) ;" << endl ;
    str << "    }" << endl ;
    str << "    ofstream htmlFile(htmlFileName.c_str()) ;" << endl ;
    str << "  htmlFile << \"<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\" << endl ;" << endl ;
    str << "    htmlFile << \"\" << endl ;" << endl ;
    str << "  htmlFile << \"<html>\" << endl ;" << endl ;
    str << "  htmlFile << \"<head>\" << endl ;" << endl ;
    str << "  htmlFile << \"<script src='http://transp-or2.epfl.ch/biogeme/sorttable.js'></script>\" << endl ;" << endl ;
    str << "  htmlFile << \"<meta http-equiv='content-type' content='text/html;charset=ISO-8859-1'>\" << endl ;" << endl ;
    str << "  htmlFile << \"<title>\"<< htmlFileName <<\" - Simulation report from \" << bioVersion::the()->getVersionInfoDate() << \"</title>\" << endl ;" << endl ;
    str << "  htmlFile << \"<meta name='keywords' content='biogeme, discrete choice, random utility'>\" << endl ;" << endl ;
    str << "  htmlFile << \"<meta name='description' content='Report from \" << bioVersion::the()->getVersionInfoDate() << \"'>\" << endl ;" << endl ;
    str << "  htmlFile << \"<meta name='author' content='Michel Bierlaire'>\" << endl ;" << endl ;
    str << "  htmlFile << \"<style type=text/css>\" << endl ;" << endl ;
    str << "  htmlFile << \"<!--table\" << endl ;" << endl ;
    str << "  htmlFile << \".biostyle\" << endl ;" << endl ;
    str << "  htmlFile << \"	{font-size:10.0pt;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-weight:400;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-style:normal;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-family:Courier;}\" << endl ;" << endl ;
    str << "  htmlFile << \".boundstyle\" << endl ;" << endl ;
    str << "  htmlFile << \"	{font-size:10.0pt;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-weight:400;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-style:normal;\" << endl ;" << endl ;
    str << "  htmlFile << \"	font-family:Courier;\" << endl ;" << endl ;
    str << "  htmlFile << \"        color:red}\" << endl ;" << endl ;
    str << "  htmlFile << \"-->\" << endl ;" << endl ;
    str << "  htmlFile << \"</style>\" << endl ;" << endl ;
    str << "  htmlFile << \"</head>\" << endl ;" << endl ;
    str << "  htmlFile << \"\" << endl ;" << endl ;
    str << "  htmlFile << \"<body bgcolor='#ffffff'>\" << endl ;" << endl ;
    str << "" << endl ;
    str << "  htmlFile << \"<h1>Simulation report</h1>\" << endl ;" << endl ;
    str << "  htmlFile << \"<h1>\" << bioVersion::the()->getVersionInfoDate() << \"</h1>\" << endl ;" << endl ;
    str << "  htmlFile << \"<h2>\" << bioVersion::the()->getVersionInfoAuthor() << \"</h2>\" << endl ;" << endl ;
    str << "" << endl ;
    str << "  htmlFile << \"<p>This file has automatically been generated.</p>\" << endl ;" << endl ;
    str << "  patAbsTime now ;" << endl ;
    str << "  htmlFile << \"<p> \" << now.getTimeString(patTsfFULL) << \"</p>\" << endl ;" << endl ;
    str << "  htmlFile << endl ;" << endl ;


    str << "  htmlFile << \"<table border='1' class='sortable'>\" << endl ;" << endl ;
    stringstream firstLine ;
    firstLine << "<tr class=biostyle><th></th>" ;
    for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
	 i != theExpressions.end() ;
	 ++i) {
      firstLine << "<th>" << i->first << "</th>";
    }
    firstLine << "</tr>" ;
    DEBUG_MESSAGE("First line: " << firstLine.str()) ;

    str << " htmlFile << \"" << firstLine.str() << "\" << endl ;" ;
    str <<  endl ;

    str << "patULong __row = 0 ;" << endl ; 

    str << "for(" << prefix << "theIter_" << n <<  "_" << uniqueId << "->first() ; " << endl ;
    str << "    !" << prefix << "theIter_" << n <<  "_" << uniqueId << "->isDone() ;" << endl ;
    str << "    " << prefix << "theIter_" << n <<  "_" << uniqueId << "->next()) {" << endl ;
    if (theIteratorInfo->isRowIterator()) {
      str << "  patVariables* __x = " << prefix << "theIter_" << n <<  "_" << uniqueId << "->currentItem() ;" 
	  << endl ;
    }
    else if (theIteratorInfo->isMetaIterator()) {
      //      DEBUG_MESSAGE("  " << prefix << "aSpan"<<uniqueId<<" = " << prefix << "theIter_" << n <<  "_" << uniqueId << "->currentItem() ;") ;

      str <<"  " << prefix << "aSpan"<<uniqueId<<" = " << prefix << "theIter_" << n <<  "_" << uniqueId << "->currentItem() ;" << endl ;  
    }
    else if (theIteratorInfo->isDrawIterator()) {
      str << "  __draw = " << prefix << "theIter_" << n <<  "_" << uniqueId << "->currentItem() ;" 
	  << endl ;
    }


    
    str << "      htmlFile << \"<tr><td>\"<< ++__row <<\"</td>\" ;" << endl ;
    for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
	 i != theExpressions.end() ;
	 ++i) {
      str << "      htmlFile << \"<td>\" ;" << endl ;
      
      if (!i->second->isSimple()) {
	str << i->second->getCppCode(prefix, err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return patString() ;
	}
      }
      str << "  htmlFile << "  ;
      
      if (i->second->isSimple()) {
	str << i->second->getSimpleCppCode(err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return patString() ;
	}
      }
      else {
	str << i->second->getCppResult(prefix) ;
	
      }
      str << " ; " << endl ;
      str << "      htmlFile << \"</td>\" ;" << endl ;
    }
    str << "htmlFile << endl ;" << endl ;
    str << "      htmlFile << \"</tr>\" ;" << endl ;
    str << "  }" << endl ;
    str << "      htmlFile << \"</table>\" ;" << endl ;

    str << "   htmlFile.close() ;" << endl ;

    str << "    GENERAL_MESSAGE(\"Output file \" << htmlFileName << \" has been generated\") ;" << endl ;

    codeMustBeGenerated = patFALSE ;
    DEBUG_MESSAGE("Code generated: return the string") ;
    return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
  
}

void bioArithPrint::generateGslFunctions(ostream& str_h,patString prefix,patError*& err) {
  for (map<patString,bioExpression*>::const_iterator i = theExpressions.begin() ;
       i != theExpressions.end() ;
       ++i) {
    
    i->second->generateGslFunctions(str_h,prefix,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
  }

}
