//-*-c++-*------------------------------------------------------------
//
// File name : bioArithProd.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Wed Jul  8 15:07:35  2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patError.h"

#include "bioArithProd.h"
#include "bioArithMult.h"
#include "bioArithDivide.h"
#include "bioArithSum.h"
#include "bioLiteralRepository.h"

bioArithProd::bioArithProd(bioExpression* par,
			   bioExpression* left,
                         bioIteratorInfo * it) 
  : bioArithIterator(par,left,it)  {

}


bioArithProd::~bioArithProd() {}


bioExpression::bioOperatorType bioArithProd::getOperatorType() const {
  return bioExpression::PROD_OP ;
}


patString bioArithProd::getOperatorName() const {
  return patString("Prod") ;
}


bioExpression* bioArithProd::getDerivative(patULong aLiteralId, patError*& err) const {

  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* theProd = new bioArithProd(NULL,leftChild,theIteratorInfo) ;

  bioExpression* leftValue = leftChild->getDeepCopy(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* leftResult = leftChild->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* ratio = new bioArithDivide(NULL,leftResult,leftValue) ;
  bioExpression* theSum = new bioArithSum(NULL,ratio,theIteratorInfo) ;
  bioExpression* result = new bioArithMult(NULL,theProd,theSum) ;
  
  return result ;
}


bioArithProd* bioArithProd::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  bioArithProd* theNode = 
    new bioArithProd(NULL,leftClone,theIteratorInfo) ;
  leftClone->setParent(theNode) ;
  
  return theNode ;
}

patBoolean bioArithProd::isSum() const {
  return patFALSE ;
}

patBoolean bioArithProd::isProd() const {
  return patTRUE ;
}

patString bioArithProd::getExpressionString() const {
  stringstream str ;
  str << '$P' ;
  str << theIteratorInfo->iteratorName ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}
