//-*-c++-*------------------------------------------------------------
//
// File name : bioArithSequence.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Aug 31 13:34:57 2009
//
//--------------------------------------------------------------------

#ifndef bioArithSequence_h
#define bioArithSequence_h

#include "bioExpression.h"

/*!  This class gathers a list of expressions such that the result of
one expression can be utilized by the subsequent expressions without
being reevaluated.
*/

class bioArithSequence : public bioExpression {

public: 
  // Left is evaluated first, and its result is used by right.  With
  // this ctor, an new literal is created. It is typically called when
  // simplification is performed
  bioArithSequence(bioExpression* par, 
		   bioExpression* left, 
		   bioExpression* right,
		   patError*& err) ;
  // This ctor recycle an existing literal. It is typically called when derivatives are computed.
  bioArithSequence(bioExpression* par, 
		   bioExpression* left, 
		   bioExpression* right,
		   patULong id, 
		   patError*& err) ;
  ~bioArithSequence() ;

public:
  virtual bioOperatorType getOperatorType() const ;
  virtual patString getOperatorName() const ;
  virtual patReal getValue(patError*& err) const ;
  virtual bioExpression* getDerivative(patULong aLiteralId, 
				       patError*& err) const ;
  virtual bioExpression* getDeepCopy(patError*& err) ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual patString getSimpleCppCode(patError*& err) ;
  virtual patBoolean isSimple() const ;
  virtual patString getExpression(patError*& err) const ;
  virtual patULong getNumberOfOperations() const ;
  virtual patString getExpressionString() const  ;
  virtual patULong countExpressions(map<patString,patULong>* theExpressions, 
				    map<patString,list<patULong> >* theIds,
				    map<patString,list<bioExpression*> >* thePointers,
				    patError*& err)  ; 
  virtual patBoolean isSequence() const ;
  virtual void simplifyLowerIteratorOrSequence(patError*& err) ;

private:
  // THe ID  of the literal created.
  patULong theId ;
};


#endif
