//-*-c++-*------------------------------------------------------------
//
// File name : bioArithSum.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Jun 16 12:47:18  2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patError.h"

#include "bioArithSum.h"
#include "bioArithBinaryPlus.h"
#include "bioLiteralRepository.h"

/*!
*/
bioArithSum::bioArithSum(bioExpression* par,
                         bioExpression* left,
                         bioIteratorInfo * it) 
  : bioArithIterator(par,left,it) {

}

bioArithSum::~bioArithSum() {}

bioExpression::bioOperatorType bioArithSum::getOperatorType() const {
  return bioExpression::SUM_OP ;
}

patString bioArithSum::getOperatorName() const {
  return patString("Sum") ;
}

bioExpression* bioArithSum::getDerivative(patULong aLiteralId, patError*& err) const {

  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* leftResult = leftChild->getDerivative(aLiteralId,err) ;

  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  // Note that only the pointer to iterator info is copied
  bioExpression* result = new bioArithSum(NULL,leftResult,theIteratorInfo) ;
  return result ;
}


bioArithSum* bioArithSum::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  bioArithSum* theNode = 
    new bioArithSum(NULL,leftClone,theIteratorInfo) ;
  leftClone->setParent(theNode) ;
  
  return theNode ;
}



patBoolean bioArithSum::isSum() const {
  return patTRUE ;
}

patBoolean bioArithSum::isProd() const {
  return patFALSE ;
}

patString bioArithSum::getExpressionString() const {
  stringstream str ;
  str << '$S' ;
  str << theIteratorInfo->iteratorName ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}
