//-*-c++-*------------------------------------------------------------
//
// File name : bioFixedParameter.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Apr 28 10:54:02 2009
//
//--------------------------------------------------------------------

#include <sstream>
#include "bioFixedParameter.h"

patReal bioFixedParameter::getValue() const {
  return currentValue ;
}
  
void bioFixedParameter::setValue(patReal val) {
  currentValue = val ;
}

bioFixedParameter::bioFixedParameter(patString theName, 
				     patULong id,
				     patReal def, 
				     patReal lb, 
				     patReal ub, 
				     patBoolean f,
				     patString desc) :
  bioLiteral(theName,id),
  defaultValue(def),
  currentValue(def),
  lowerBound(lb),
  upperBound(ub),
  isFixed(f),
  description(desc) {
  
}

bioFixedParameter::bioFixedParameter(patString theName, 
				     patULong id,
				     patReal def):
  bioLiteral(theName,id),
  defaultValue(def),
  currentValue(def),
  lowerBound(-1.0e20),
  upperBound(1.0e20),
  isFixed(patFALSE),
  description(theName) {

}

bioLiteral::bioLiteralType bioFixedParameter::getType() const {
  return PARAMETER ;
}

ostream& operator<<(ostream &str, const bioFixedParameter& x) {
  str << "x[" ;
  str << x.theParameterId << "]=" << x.name << " (" << x.defaultValue << "," 
      <<  x.currentValue << ") [" << x.lowerBound << "," << x.upperBound << "] " ;
  if (x.isFixed) {
    str << "FIXED " ;
  }
  else {
    str << "FREE " ;
  }
  str << "ID=" << x.uniqueId ;

  return str ;
}

patReal bioFixedParameter::getLowerBound() const {
  return lowerBound ;
}

patReal bioFixedParameter::getUpperBound() const {
  return upperBound ;
}

patBoolean bioFixedParameter::isFixedParam() const {
  return (isFixed != 0) ;
}

patString bioFixedParameter::printPythonCode(patBoolean estimation) const {
  stringstream str ;
  if (estimation) {
    str << name << " = Beta(";
    str << "'" << name << "'," ;
    str << currentValue << "," ; 
    str << lowerBound << "," ; 
    str << upperBound << "," ; 
    if (isFixed) {
      str << "1" ; 
    }
    else {
      str << "0" ; 
    }
    str << " )" << endl ;
  }
  else {
    str << name << " = " << currentValue << endl ;
  }
  return patString(str.str());
}
