//-*-c++-*------------------------------------------------------------
//
// File name : bioIteratorInfo.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Jul 13 15:01:06  2009
//
//--------------------------------------------------------------------

#ifndef bioIteratorInfo_h
#define bioIteratorInfo_h

#include "patError.h"
#include "patString.h"

/*!

*/
typedef enum {
  META,
  ROW,
  DRAW,
  UNDEFINED
} bioIteratorType ;

class bioIteratorInfo {

  friend class bioIteratorInfoRepository ;
  friend ostream& operator<<(ostream &str, const bioIteratorInfo& x) ;
public:
  bioIteratorInfo() ;
  patBoolean operator==(const bioIteratorInfo &other) const ;
  patBoolean operator!=(const bioIteratorInfo &other) const ;
  // Ctor for iterators at the file level
  bioIteratorInfo(patString aFile, 
                  patString childIteratorName,
                  patString anIndex, 
                  bioIteratorType aType) ;
  
  // Ctor for iterators on another iterator
  bioIteratorInfo(bioIteratorInfo* i,
		  patString childIteratorName,
                  patString anIndex, 
                  bioIteratorType aType) ;

  // Ctor for draws iterator
  bioIteratorInfo(patString aFile) ;

  patString getInfo() ;

  patString getCppType(patError*& err) const ;

  void computePointers(patULong sampleSize,patError*& err) ;

  patBoolean isTopIterator() const;
  patBoolean isRowIterator() const ;
  patBoolean isMetaIterator() const ;
  patBoolean isDrawIterator() const ;

// private:
  patString iteratorName ;
  patString datafile ;
  patString indexName ;
  patString childIteratorName ;
  bioIteratorInfo* theParent ;
  bioIteratorType type ;
  // Contains the IDs of the first row of each block
  vector<patULong> mapIds ; 

  // Contains for each row a pointer to the first row of the next
  // block, or patBadId if the row does not correspond to the
  // beginning of a block.
  vector<patULong> rowPointers ;

  // Used for the generation of the c++ code
  patString idOfCurrentNodeInExpression ;  
};


#endif
