//-*-c++-*------------------------------------------------------------
//
// File name : bioIteratorInfoRepository.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Jul 14 13:25:19 2009
//
//--------------------------------------------------------------------

#include <sstream>
#include "patDisplay.h"
#include "bioIteratorInfoRepository.h"
#include "patErrMiscError.h"
#include "patErrOutOfRange.h"

bioIteratorInfoRepository::bioIteratorInfoRepository(): topIterator("") {
}

bioIteratorInfoRepository* bioIteratorInfoRepository::the() {
  static bioIteratorInfoRepository* singleInstance = NULL ;
  if (singleInstance == NULL) {
    singleInstance = new bioIteratorInfoRepository ;
  }
  return singleInstance ;
}

void bioIteratorInfoRepository::addIteratorInfo(patString name, 
						bioIteratorInfo ii, 
						patError*& err) {
  ii.iteratorName = name ;
  pair<patString,bioIteratorInfo> p(name,ii) ;
  theRepository.insert(p) ;
  if (ii.isTopIterator() && !ii.isDrawIterator()) {
    if (topIterator != "") {
      stringstream str ;
      str << "There are (at least) two top iterators: " << topIterator << " and " << name ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()) ;
      return  ;
    }
    else {
      topIterator = name ;
    }
  }
}

bioIteratorInfo* bioIteratorInfoRepository::getIteratorInfo(patString name) {


  map<patString,bioIteratorInfo>::iterator found = theRepository.find(name) ;
  if (found == theRepository.end()) {
    return NULL ;
  }
  return &(found->second) ;
}


patString bioIteratorInfoRepository::debug(patError*& err) {
  stringstream str ;
  str << "List of iterators" << endl ;
  for (map<patString,bioIteratorInfo>::iterator i = theRepository.begin() ;
       i != theRepository.end() ;
       ++i) {
    str << i->second << endl ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString() ;
    }
  }
  return patString(str.str()) ;
}


//map<patString,bioIteratorInfo>::const_iterator 
//bioIteratorInfoRepository::getRepositoryIterator() const {
//  return theRepository.begin() ;
//}

map<patString,bioIteratorInfo> bioIteratorInfoRepository::getRepository() const {
  return theRepository ;
}

void bioIteratorInfoRepository::addNewRowId(patString indexName, patULong rowNumber) {
  for (map<patString,bioIteratorInfo>::iterator i = theRepository.begin() ;
       i != theRepository.end() ;
       ++i) {
    if (i->second.indexName == indexName) {
      i->second.mapIds.push_back(rowNumber) ;
    }
  }
}

void bioIteratorInfoRepository::computeRowPointers(patULong sampleSize, 
					 patError*& err) {
  for (map<patString,bioIteratorInfo>::iterator i = theRepository.begin() ;
       i != theRepository.end() ;
       ++i) {
    i->second.computePointers(sampleSize,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
  }

  
  
}

patBoolean bioIteratorInfoRepository::isRowIterator(patString name) {
  bioIteratorInfo* theIter = getIteratorInfo(name) ;
  if (theIter == NULL) {
    return patFALSE ;
  }
  return (theIter->type == ROW) ;
  
}

bioIteratorSpan bioIteratorInfoRepository::getSublist(patULong index, patError*& err) {
  if (index >= subListsForThreads.size()) {
    err = new patErrOutOfRange<patULong>(index,0,subListsForThreads.size()-1) ;
    WARNING(err->describe()) ;
    return bioIteratorSpan() ;
  }
  return subListsForThreads[index] ;
}

void bioIteratorInfoRepository::generateSublistsForThreads(patULong dataSize, patULong numberOfThreads,patError*& err) {

  // Rows spanned by the top iterator. The thread spans should
  // correspond with these values.

  bioIteratorInfo theTopIterator = theRepository[topIterator] ;

  if (!theTopIterator.isTopIterator()) {
    return ;
  }
  vector<patULong> mapIds = theTopIterator.mapIds ;
  mapIds.push_back(dataSize) ;
  DEBUG_MESSAGE("Generate iterators for " << numberOfThreads << " threads") ;
  patULong approximateSizeOfEachBlock = ceil(patReal(dataSize)/patReal(numberOfThreads)) ;
  DEBUG_MESSAGE("Approximate size of each block: " << approximateSizeOfEachBlock) ;
  patULong currentThread = 1 ;
  patULong beginCurrentThread = 0 ;
  patULong sizeCurrentThread = 0 ;
  for (patULong i = 0 ; i < mapIds.size()-1 ; ++i) {
    sizeCurrentThread += mapIds[i+1]-mapIds[i] ;
    if (sizeCurrentThread >= approximateSizeOfEachBlock) {
      stringstream str ;
      str << "Thread " << currentThread ;
      bioIteratorSpan aSpan(str.str(),beginCurrentThread,mapIds[i+1]) ;
      DEBUG_MESSAGE("Create thread: " << aSpan) ;
      subListsForThreads.push_back(aSpan) ;
      ++currentThread ;
      beginCurrentThread = mapIds[i+1] ;
      sizeCurrentThread = 0 ;
    }
  }
  if (currentThread <= numberOfThreads) {
    stringstream str ;
    str << "Thread " << currentThread ;
    bioIteratorSpan aSpan(str.str(),beginCurrentThread,dataSize) ;
    DEBUG_MESSAGE("Create thread: " << aSpan) ;
    subListsForThreads.push_back(aSpan) ;
  }


}

