//-*-c++-*------------------------------------------------------------
//
// File name : bioMain.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri Jul 17 16:41:28 2009
//
//--------------------------------------------------------------------

#include <Python.h>
#include <iomanip>
#include "patErrNullPointer.h"
#include "patTimeInterval.h"
#include "bioMain.h"
#include "bioRandomDraws.h"
#include "bioParameters.h"
#include "bioSample.h"
#include "bioRowIterator.h"
#include "patDisplay.h"
#include "patFileNames.h"
#include "bioVersion.h"
#include "bioModelParser.h"
#include "trBounds.h"
#include "bioMinimizationProblem.h"
#include "trSimpleBoundsAlgo.h"
#include "patCfsqp.h"
#include "patSolvOpt.h"
#include "patDonlp2.h"
#include "bioRawResults.h"
#include "bioReporting.h"
#include "bioStatistics.h"
#include "bioSimulation.h"
#include "bioConstraints.h"
#include "bioIterationBackup.h"

bioMain::bioMain() : theFunction(NULL),theSample(NULL), theModel(NULL), theStatistics(NULL), theConstraints(NULL), theRawResults(NULL) {
  WARNING("Parameter management not yet implemented. Default values are used") ;


//   theTrParameters.eta1 =  patParameters::the()->getBTREta1() ;
//   theTrParameters.eta2 =  patParameters::the()->getBTREta2() ; ;
//   theTrParameters.gamma2 = patParameters::the()->getBTRGamma2() ; ;
//   theTrParameters.beta = patParameters::the()->getBTRIncreaseTRRadius() ; ;
//   theTrParameters.maxIter =  patParameters::the()->getBTRMaxIter() ; ;
//   theTrParameters.initQuasiNewtonWithTrueHessian = patParameters::the()->getBTRInitQuasiNewtonWithTrueHessian() ;
//   theTrParameters.significantDigits = patParameters::the()->getBTRSignificantDigits(); ;
//   theTrParameters.usePreconditioner = patParameters::the()->getBTRUsePreconditioner(); ;
//   theTrParameters.maxTrustRegionRadius = patParameters::the()->getBTRMaxTRRadius() ;
//   theTrParameters.typicalF = patParameters::the()->getBTRTypf() ;
//   theTrParameters.tolerance = patParameters::the()->getBTRTolerance() ;
//   theTrParameters.toleranceSchnabelEskow =  patParameters::the()->getTolSchnabelEskow() ;
//   theTrParameters.exactHessian = patParameters::the()->getBTRExactHessian() ;
//   theTrParameters.cheapHessian = patParameters::the()->getBTRCheapHessian() ;
//   theTrParameters.initRadius = patParameters::the()->getBTRInitRadius() ;
//   theTrParameters.minRadius = patParameters::the()->getBTRMinTRRadius() ;
//   theTrParameters.armijoBeta1 = patParameters::the()->getBTRArmijoBeta1() ;
//   theTrParameters.armijoBeta2 = patParameters::the()->getBTRArmijoBeta2() ;
//   theTrParameters.stopFileName = patParameters::the()->getgevStopFileName() ;
//   theTrParameters.startDraws =  patParameters::the()->getBTRStartDraws() ; ;
//   theTrParameters.increaseDraws =  patParameters::the()->getBTRIncreaseDraws() ; ;
//   theTrParameters.maxGcpIter = patParameters::the()->getBTRMaxGcpIter() ;
//   theTrParameters.fractionGradientRequired =  patParameters::the()->getTSFractionGradientRequired() ;
//   theTrParameters.expTheta = patParameters::the()->getTSExpTheta() ;
//   theTrParameters.infeasibleCgIter = patParameters::the()->getBTRUnfeasibleCGIterations() ;
//   theTrParameters.quasiNewtonUpdate =  patParameters::the()->getBTRQuasiNewtonUpdate() ;
//   theTrParameters.kappaUbs =  patParameters::the()->getBTRKappaUbs() ;
//   theTrParameters.kappaLbs =  patParameters::the()->getBTRKappaLbs() ;
//   theTrParameters.kappaFrd =   patParameters::the()->getBTRKappaFrd() ;
//   theTrParameters.kappaEpp =  patParameters::the()->getBTRKappaEpp()  ;
  
//   theSolvoptParameters.errorArgument = patParameters::the()->getsolvoptErrorArgument() ;
//   theSolvoptParameters.errorFunction = patParameters::the()->getsolvoptErrorFunction() ;
//   theSolvoptParameters.maxIter =  patParameters::the()->getsolvoptMaxIter() ;
//   theSolvoptParameters.display =  patParameters::the()->getsolvoptDisplay() ;
  
//   theDonlp2Parameters.epsx = patParameters::the()->getdonlp2Epsx() ;
//   theDonlp2Parameters.delmin = patParameters::the()->getdonlp2Delmin() ;
//   theDonlp2Parameters.smallw = patParameters::the()->getdonlp2Smallw() ;
//   theDonlp2Parameters.epsdif = patParameters::the()->getdonlp2Epsdif() ;
//   theDonlp2Parameters.nreset = patParameters::the()->getdonlp2NReset() ;
//   theDonlp2Parameters.stopFileName = patParameters::the()->getgevStopFileName() ;
}

void bioMain::setFunction(bioPythonWrapper* f) {
  theFunction = f ;
}

void bioMain::setStatistics(bioStatistics* s) {
  theStatistics = s ;
}

void bioMain::setConstraints(bioConstraints* c) {
  theConstraints= c ;
}

void bioMain::run(int argc, char *argv[]) {

  patError* err(NULL) ;
  
  // Reads the sample file

  patString sampleFile("sample.dat") ;
  if (argc >= 3) {
    sampleFile = patString(patString(argv[2])) ;
  }
  //  DEBUG_MESSAGE("Sample file: " << sampleFile) ;
  
  theSample = new bioSample(sampleFile,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    exit(-1) ;
  }


  // Init Python
  Py_Initialize() ;

  PyRun_SimpleString("import sys") ;
  PyRun_SimpleString("sys.path.append('./module/')") ;



  // Get model name


  patString fn(argv[1]) ;

  //  DEBUG_MESSAGE("Model file: " << argv[1]) ;
  patString pythonFileExtension(".py") ;
  int extensionPos = fn.size()-pythonFileExtension.size() ;

  // Remove .py extension if present
  if (fn.find(pythonFileExtension, extensionPos)!=string::npos) {
    fn.erase(extensionPos) ;
  }

  patFileNames::the()->setModelName(fn) ;

  GENERAL_MESSAGE("READ MODEL FROM " << fn) ;
  bioModelParser* parser = new bioModelParser(fn) ;
  parser->setSampleFile(sampleFile) ;
  theModel = parser->readModel(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    exit(-1) ;
  }
  GENERAL_MESSAGE("MODEL READ") ;
  theSample->prepare(theModel->getExcludeExpr(),err) ;
  if (err != NULL) {
    WARNING(err->describe()); 
    exit(-1) ;
  }

  theReport.computeFromSample(theSample,err) ;

  DEBUG_MESSAGE("Sample of size: " << theSample->size()) ;


  ofstream po("__parametersUsed.py") ;
  po << bioParameters::the()->printPythonCode() << endl;
  po.close() ;


  bioRandomDraws::the()->populateListOfIds(theSample,err) ;
  if (err != NULL) {
    WARNING(err->describe()); 
    exit(-1) ;
  }
  bioRandomDraws::the()->generateDraws(err) ;
  if (err != NULL) {
    WARNING(err->describe()); 
    exit(-1) ;
  }

  statistics(err) ;
  
  if (theModel->mustEstimate()) {
    estimate(err) ;
    if (err != NULL) {
      WARNING(err->describe()); 
      exit(-1) ;
    }
    patString htmlFile = patFileNames::the()->getHtmlFile(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    
    theReport.writeHtml(htmlFile,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    
  }

  if (theModel->mustSimulate()) {
    //For simulation, only one thread is used
    int nt = bioParameters::the()->getValueInt("numberOfThreads",err) ; 
    if (err != NULL) {
      WARNING(err->describe()); 
      exit(-1) ;
    }
    if (nt > 1) {
      stringstream str ;
      str << "Only one thread can be used for simulation. Add the following line in the specification file:" << endl ;
      str << "BIOGEME_OBJECT.PARAMETERS['numberOfThreads'] = \"1\"" << endl ;
      err = new patErrMiscError(str.str()) ;
      WARNING(err->describe()); 
      exit(-1) ;

    }
    simulate(err) ;
    if (err != NULL) {
      WARNING(err->describe()); 
      exit(-1) ;
    }
  }


}

void bioMain::estimate(patError*& err) {

  if (theSample == NULL) {
    err = new patErrMiscError("No sample has been loaded. Biogeme cannot estimate the parameters") ;
    WARNING(err->describe()) ;
    return ;
  }

  theFunction->setSample(theSample,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }


  // Bounds

  patVariables l = bioLiteralRepository::the()->getLowerBounds() ;
  patVariables u = bioLiteralRepository::the()->getUpperBounds() ;
  trBounds theBounds(l,u) ;
  
  patVariables beta = bioLiteralRepository::the()->getBetaValues(patFALSE) ;
  patBoolean success ;
  patReal initLikelihood(0.0) ;
  patBoolean computeInit = (bioParameters::the()->getValueInt("computeInitLoglikelihood",err) != 0) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  
  patAbsTime beg ;
  patAbsTime endc ;

  
  if (computeInit) {
    beg.setTimeOfDay() ;
    initLikelihood = -theFunction->getFunction(&beta,&success,err) ;
    endc.setTimeOfDay() ;
    patTimeInterval til0(beg,endc) ;
    GENERAL_MESSAGE("Init. log-likelihood: " << initLikelihood << " ["<<til0.getLength()<<"]") ;
  }
  patBoolean checkDeriv = (bioParameters::the()->getValueInt("checkDerivatives",err) != 0) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  if (checkDeriv) {
    patVariables grad(beta.size(),0.0) ;
    beg.setTimeOfDay() ;
    theFunction->getGradient(&beta,&grad,&success,err) ;
    endc.setTimeOfDay() ;
    patTimeInterval tig(beg,endc) ;
    patVariables finDiffGrad(beta.size(),0.0) ;
    beg.setTimeOfDay() ;
    theFunction->getFinDiffGradient(&beta,&finDiffGrad,&success,err) ;
    endc.setTimeOfDay() ;
    patTimeInterval tigfd(beg,endc) ;

    
    GENERAL_MESSAGE("Analytical\t\tFin. diff.\tError\tName") ;
    for (patULong i = 0 ; i < grad.size() ; ++i) {
      patString name = bioLiteralRepository::the()->getBetaName(i,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      GENERAL_MESSAGE( setprecision(7) << setiosflags(ios::scientific|ios::showpos) << grad[i] << '\t' << finDiffGrad[i] << '\t' << setprecision(2) << 100*(grad[i]-finDiffGrad[i])/finDiffGrad[i]<<"% \t" << name) ;
    }

  }

  // Optimization problem

  bioMinimizationProblem theProblem(theFunction, theBounds,theTrParameters) ;
  

  // Constraints
  if (theConstraints != NULL) {
    theConstraints->addConstraints() ;
    vector<bioConstraintWrapper*>* c = theConstraints->getConstraints() ;
    for (vector<bioConstraintWrapper*>::iterator i = c->begin() ;
	 i != c->end() ;
	 ++i) {
      theProblem.addEqualityConstraint(*i) ;
    }
    
  }
  
  // Optimization algorithm

  patReal initRadius = bioParameters::the()->getValueReal("BTRInitRadius",err) ;
  if (err != NULL) {
    WARNING(err->describe()); 
  }

  beg.setTimeOfDay() ;

  patString selectedAlgo = bioParameters::the()->getValueString("optimizationAlgorithm",err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  if (theProblem.nNonTrivialConstraints() > 0 && selectedAlgo == "BIO") {
    if (patCfsqp::isAvailable) {
      selectedAlgo = "CFSQP" ;
      GENERAL_MESSAGE("The problem has " << theProblem.nNonTrivialConstraints() << " constraints. Algorithm BIO cannot be used. CFSQP is used instead.") ; 
    }
    else {
      selectedAlgo = "DONLP2" ;
      GENERAL_MESSAGE("The problem has " << theProblem.nNonTrivialConstraints() << " constraints. Algorithm BIO cannot be used. DONLP2 is used instead.") ; 
    }

    patULong nle = theProblem.nNonLinearEq() ;
    for (patULong i = 0 ; i < nle ; ++i) {
      trFunction* f = theProblem.getNonLinEquality(i,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      patBoolean success ;
      patReal value = f->getFunction(&beta,&success,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
  }


  if (!theProblem.isFeasible(beta,err)) {
    WARNING("Starting point not feasible") ;
  }

  trNonLinearAlgo* algo ;
  if (selectedAlgo == "BIO") {
    patIterationBackup* aBackup = new bioIterationBackup() ;
    algo = new trSimpleBoundsAlgo(&theProblem,
				  beta,
				  theTrParameters,
				  aBackup,
				  err) ;
    
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
  }

  else if (selectedAlgo == "CFSQP") {
      
    DETAILED_MESSAGE("Use CFSQP") ;
      
    patIterationBackup* aBackup = new bioIterationBackup() ;
    patCfsqp* algoCfsqp = new patCfsqp(aBackup,&theProblem) ;

    int mode = bioParameters::the()->getValueInt("cfsqpMode",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    int iprint = bioParameters::the()->getValueInt("cfsqpIprint",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    int miter = bioParameters::the()->getValueInt("cfsqpMaxIter",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    double eps = bioParameters::the()->getValueReal("cfsqpEps",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    double epseqn = bioParameters::the()->getValueReal("cfsqpEpsEqn",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    double udelta = bioParameters::the()->getValueReal("cfsqpUdelta",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    patString stopFile = bioParameters::the()->getValueString("stopFileName",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    
    algoCfsqp->setParameters(mode,
			     iprint,
			     miter,
			     eps,
			     epseqn,
			     udelta,
			     stopFile) ;
    algo = algoCfsqp ;
    if (algo == NULL) {
      err = new patErrNullPointer("patCfsqp") ;
      return ;
    }
    algo->defineStartingPoint(beta) ;
  }
  else if (selectedAlgo == "SOLVOPT") {
    patIterationBackup* aBackup = new bioIterationBackup() ;
    algo = new patSolvOpt(theSolvoptParameters,&theProblem) ;
    if (algo == NULL) {
      err = new patErrNullPointer("patSolvOpt") ;
      WARNING(err->describe()) ;
      return ;
    }
    algo->setBackup(aBackup) ;
    algo->defineStartingPoint(beta) ;
  }
  else if (selectedAlgo == "DONLP2") {
    patIterationBackup* aBackup = new bioIterationBackup() ;
    algo = new patDonlp2(theDonlp2Parameters,&theProblem) ;
    if (algo == NULL) {
      err = new patErrNullPointer("patDonlp2") ;
      WARNING(err->describe()) ;
      return ;     
    }
    algo->setBackup(aBackup) ;
    algo->defineStartingPoint(beta) ;
  }
  else {
    stringstream str ;
    str << "Unknown algorithm: " << selectedAlgo ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return ;
  }

  patString algoName(algo->getName()) ;
  
  DEBUG_MESSAGE("About to run algorithm: " << algoName) ;
  
  patString diag = algo->run(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  endc.setTimeOfDay() ;
  patTimeInterval ti0(beg,endc) ;
  
  DETAILED_MESSAGE("Run time interval: " << ti0 ) ;
  DETAILED_MESSAGE("Run time: " << ti0.getLength()) ;

  patVariables solution = algo->getSolution(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  DEBUG_MESSAGE("Solution: " << solution) ;
  bioLiteralRepository::the()->setBetaValues(solution,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  patString pythonEstimated = patFileNames::the()->getPythonEstimated(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  ofstream pyEst(pythonEstimated.c_str()) ;
  pyEst << "# This file has automatically been generated" << endl ; 
  pyEst << "# " << bioVersion::the()->getVersionInfoDate() << endl ;
  pyEst << "# " << bioVersion::the()->getVersionInfoAuthor() << endl ;
  patAbsTime now ;
  now.setTimeOfDay() ;
  pyEst << "# " << now.getTimeString(patTsfFULL) << "</p>" << endl ;
  pyEst << "#" << endl ;
  pyEst << bioLiteralRepository::the()->printFixedParameters(patTRUE) << endl ;
//   pyEst << "# Code to use for simulation" << endl ;
//   pyEst << bioLiteralRepository::the()->printFixedParameters(patFALSE) << endl ;
  pyEst.close() ;
  

  GENERAL_MESSAGE("Python code for the estimated parameters: " << endl << bioLiteralRepository::the()->printFixedParameters()) ;
  GENERAL_MESSAGE("File " << pythonEstimated << " created") ;

  theRawResults = new bioRawResults(&theProblem,solution,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  theReport.computeEstimationResults(theRawResults,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }


//   // Statistics
  
//   vector<pair<patString,patReal> > stats = theModel->getStatistics() ;
//   if (err != NULL) {
//     WARNING(err->describe()); 
//     return ;
//   }
//   for (vector<pair<patString,patReal> >::iterator i = stats.begin() ;
//        i != stats.end() ;
//        ++i) {
//     theReport.addStatistic(i->first,i->second) ;
//   }
  
  theReport.setDiagnostic(diag) ;
  theReport.setIterations(algo->nbrIter()) ;
  theReport.setRunTime(ti0.getLength()) ;
  theReport.setInitLL(initLikelihood) ;
}


void bioMain::simulate(patError*& err) {

  if (theSample == NULL) {
    err = new patErrMiscError("No sample has been loaded. Biogeme cannot estimate the parameters") ;
    WARNING(err->describe()) ;
    return ;
  }

  theFunction->setSample(theSample,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  patVariables beta = bioLiteralRepository::the()->getBetaValues(patFALSE) ;
  patBoolean success ;
  patReal result = theFunction->getFunction(&beta,&success,err) ;
  


}


void bioMain::statistics(patError*& err) {

  if (theSample == NULL) {
    err = new patErrMiscError("No sample has been loaded. Biogeme cannot estimate the parameters") ;
    WARNING(err->describe()) ;
    return ;
  }

  if (theStatistics == NULL) {
    return ;
  }

  theStatistics->bio__generateStatistics(theSample, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }
  vector<pair<patString,patReal> >* theStats = theStatistics->getStatistics() ;
  for (vector<pair<patString,patReal> >::iterator i = theStats->begin() ;
       i != theStats->end() ;
       ++i) {
    theReport.addStatistic(i->first,i->second) ;
  }
}
