//-*-c++-*------------------------------------------------------------
//
// File name : bioMetaIterator.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Wed Jun 17 16:08:37  2009
//
//--------------------------------------------------------------------


#include "patDisplay.h"
#include "bioMetaIterator.h"
#include "bioIteratorInfoRepository.h"
#include "patErrMiscError.h"
#include "patErrNullPointer.h"
#include "patMath.h"

bioMetaIterator::bioMetaIterator(bioIteratorSpan aSpan, bioIteratorSpan threadSpan,patError*& err) :
  firstRow(aSpan.firstRow),
  lastRow(aSpan.lastRow) {

  iteratorInfo = bioIteratorInfoRepository::the()->getIteratorInfo(aSpan.name) ;
  if (iteratorInfo == NULL) {
    err = new patErrNullPointer("bioIteratorInfo") ;
    WARNING(err->describe()) ;
    return ;
  }
  rowPointers = &(iteratorInfo->rowPointers) ;

  if (iteratorInfo->isTopIterator()) {
    bioIteratorSpan intersect = aSpan.intersection(threadSpan) ;
    firstRow = intersect.firstRow ;
    lastRow = intersect.lastRow ;
  }
  
}


void bioMetaIterator::first() {
  currentRow = firstRow ;
}


void bioMetaIterator::next() {
  if (currentRow != patBadId) {
    if (currentRow > rowPointers->size()) {
      WARNING("ROW NUMBER OUT OF BOUNDS: " << currentRow <<  " > " << rowPointers->size()) ;
    }
    else {
      currentRow = (*rowPointers)[currentRow] ;
    }
  }
}


patBoolean bioMetaIterator::isDone() {
  if (currentRow == patBadId) {
    return patTRUE ;
  }
  return (currentRow >= lastRow) ;
}


bioIteratorSpan bioMetaIterator::currentItem() {
  bioIteratorSpan result ;
  if (iteratorInfo != NULL) {
    result.name = iteratorInfo->childIteratorName ;
  }
  
  if (!this->isDone()) {
    result.firstRow = currentRow ; 
    if (currentRow == patBadId) {
      result.lastRow = patBadId ;
    }
    else {
      result.lastRow = (*rowPointers)[currentRow] ;
    }
  } 
  else {
    result.firstRow = patBadId  ;
    result.lastRow = patBadId  ;
  }
  return result ;
}


