from biogeme import *
from mev import *

# Implements the cross nested logit model as a MEV model.
# Nests are defined as follows:
#alphaA = {1: alpha1a,
#          2: alpha2a,
#          3: alpha3a,
#          4: alpha4a,
#          5: alpha5a,
#          6: alpha6a}
#alphaB = {1: alpha1b,
#          2: alpha2b,
#          3: alpha3b,
#          4: alpha4b,
#          5: alpha5b,
#          6: alpha6b}
#nesta = MUA , alphaA
#nestb = MUB , alphaB
#nests = nesta, nestb
#


def cnl(V,nests,choice) :

    Gi = {}
    chosen = Elem(V,choice)
    for k in V:
        Gi[k] = 0.0
    for m in nests:
        biosum = 0.0
        for i,a in m[1].items():
            biosum += a**(m[0]) * exp(m[0] * (V[i]-chosen))
        for i,a in m[1].items():
            Gi[i] += (biosum**((1.0/m[0])-1.0)) * (a**m[0]) * exp((m[0]-1.0)*(V[i]-chosen))
    P = mev_shifted(V,Gi,choice) 
    return P

# The same, where availabilities are accounted for
def cnl_avail(V,availability,nests,choice) :

    Gi = {}
    chosen = Elem(V,choice)
    for k in V:
        Gi[k] = 0.0
    for m in nests:
        biosum = 0.0
        for i,a in m[1].items():
            biosum += availability[i] * a**(m[0]) * exp(m[0] * (V[i]-chosen))
        for i,a in m[1].items():
            Gi[i] += availability[i] * (biosum**((1.0/m[0])-1.0)) * (a**m[0]) * exp((m[0]-1.0)*(V[i]-chosen))
    P = mev_shifted(V,Gi,choice) 
    return P


# The same, with mu explicitly mentioned
def cnlmu(V,nests,choice,bmu) :

    Gi = {}
    chosen = Elem(V,choice)
    for k in V:
        Gi[k] = 0.0
    for m in nests:
        biosum = 0.0
        for i,a in m[1].items():
            biosum += a**(m[0]/bmu) * exp(m[0] * (V[i]-chosen))
        for i,a in m[1].items():
            Gi[i] += bmu * (biosum**((bmu/m[0])-1.0)) * (a**(m[0]/bmu)) * exp((m[0]-1.0)*(V[i]-chosen))
    P = mev_shifted(V,Gi,choice) 
    return P

def cnlmu_avail(V,availability,nests,choice,bmu) :

    Gi = {}
    chosen = Elem(V,choice)
    for k in V:
        Gi[k] = 0.0
    for m in nests:
        biosum = 0.0
        for i,a in m[1].items():
            biosum += availability[i] * a**(m[0]/bmu) * exp(m[0] * (V[i]-chosen))
        for i,a in m[1].items():
            Gi[i] += availability[i] * bmu * (biosum**((bmu/m[0])-1.0)) * (a**(m[0]/bmu)) * exp((m[0]-1.0)*(V[i]-chosen))
    P = mev_shifted(V,Gi,choice) 
    return P

